/* ***************
// bbcode.js
// Main implementation file for BBCode  <http://extensions.jedbrown.net/>
//
// Original code: BBCode Extension (http://extensions.jedbrown.net/)
// Author: Jed Brown (mozilla@jedbrown.net)
// Contributors:
//   - Ronny Perinke
//   - Cusser
//   - Patrick Wildenborg (gtafcode@patrickw.gtacrime.nl)
//   - Simon Willison
//   - Mark Wilton-Jones
//
// License:
// See license.txt file from the original .xpi.
//
// TO USE THIS CODE YOU MUST ABIDE BY THE LICENSE
****************** */
var BBCode_Color;
var BBCode_Size;
var BBCode_enableBBCode;
var BBCode_enable3dcenter;
var BBCode_enableXHTML;
var BBCode_enableHTML;
var BBCode_enableSYMBOL;
var BBCode_enableCustomCode;

var bbcode = {
	BBCode_use_newurlclip: false,	// use the new/different urlclip behavior introduced in BBCode 3DC SE 0.4.1.5
	
	prompt: function(aText) {
		var outText = new Object();
		var promptService=Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
		promptService.prompt(window, 'BBCode Prompt', aText, outText, null, {}); 
		return outText.value;
	},
	
	init: function() {
		bbcode.SetDefaultPrefs();
		bbcode.GetUserPrefs();
		try{
			var menu = document.getElementById('contentAreaContextMenu');
			menu.addEventListener('popupshowing', bbcode.showHide, false);
		}catch (e){
		}
	},
	
	showHide: function() {
		var contextUndo = document.getElementById('context-undo').hidden;
		document.getElementById('context-bbcode-main').hidden = contextUndo;    
		
		if(BBCode_enableBBCode){
			document.getElementById('context-bbcode').hidden = contextUndo;
		}else{
			document.getElementById('context-bbcode').hidden = true;
		}
		if(BBCode_enable3dcenter){
			document.getElementById('context-3DCenter').hidden = contextUndo;
		}else{
			document.getElementById('context-3DCenter').hidden = true;
		}
		if(BBCode_enableXHTML){
			document.getElementById('context-bbcode-xhtml').hidden = contextUndo;
		}else{
			document.getElementById('context-bbcode-xhtml').hidden = true;
		}
		if(BBCode_enableHTML){
			document.getElementById('context-bbcode-html').hidden = contextUndo;
		}else{
			document.getElementById('context-bbcode-html').hidden = true;
		}
		if(BBCode_enableSYMBOL){
			document.getElementById('context-bbc-symbols').hidden = contextUndo;
		}else{
			document.getElementById('context-bbc-symbols').hidden = true;
		}
		if(BBCode_enableCustomCode){
			var theNumber = bbcode.getPrefBBCodeCompose.GetIntPref('bbcode.tags.custom.number');
			if(!contextUndo && theNumber > 0 ){
				document.getElementById('context-bbc-custom').hidden = false;
			}else{
				document.getElementById('context-bbc-custom').hidden = true;
			}
		}else{
			document.getElementById('context-bbc-custom').hidden = true;
		}
		//context-bbc-symbols
	},
	
	bbcode: function(myCommand, codeType, myCustomStringIndex) {
		var clip = Components.classes["@mozilla.org/widget/clipboard;1"].createInstance(Components.interfaces.nsIClipboard);
		if (!clip)
			return false;
		
		var trans = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
		if (!trans)
			return false;
		
		trans.addDataFlavor("text/unicode");
		clip.getData(trans,clip.kGlobalClipboard);
		
		var str = null;
		var str_clipboard=new Object();
		var strLength=new Object();
		try{
			trans.getTransferData("text/unicode", str_clipboard, strLength);
			if (str_clipboard)
				str_clipboard = str_clipboard.value.QueryInterface(Components.interfaces.nsISupportsString);
		}catch (e){
			// clipboar epmty
			str_clipboard = "";
		}
		
		var theBox = document.commandDispatcher.focusedElement;
		var oPosition = theBox.scrollTop;
		var oHeight = theBox.scrollHeight;
		
		//Get Selected text (if selected)
		var startPos = theBox.selectionStart;
		var endPos = theBox.selectionEnd;
		str = theBox.value.substring(startPos, endPos);
		//alert("Str: " +str);
		
		if(codeType == "bbcode"){
			bbcode.insertAtCursorSetup(myCommand, str_clipboard, str, theBox, myCustomStringIndex);
		}else if(codeType == "html"){
			bbcode.insertAtCursorSetupHTML(myCommand, str_clipboard, str, theBox);
		}else if(codeType == "xhtml"){
			bbcode.insertAtCursorSetupXHTML(myCommand, str_clipboard, str, theBox);
		}else if(codeType == "symbol"){
			bbcode.insertAtCursor("&" + myCommand + ";");	//avoid bloat of code
		}else if(codeType == "bbcode-direct"){
			bbcode.insertAtCursor( " " + myCommand + " ");
		}else{
			alert("Error #1");
		}
		var nHeight = theBox.scrollHeight - oHeight;
		theBox.scrollTop = oPosition + nHeight;
	},
	
	insertAtCursorSetup: function(myCommand, str_clipboard, str, theBox, myCustomStringIndex) {
		var bold = "[b]";
		var boldend="[\/b]";
		var color = "[color=\"";
		var color2 = "\"]";
		var colorend = "[\/color]";
		var colorend2 = "[/color]";
		var select_flag = (str != "");
		var alertText = "";
		var theCode = "";
		switch (myCommand){
			case "quote":
				bbcode.insertAtCursorEx("[quote]" + str, "[/quote]", select_flag);
				break;
			case "quoteclip":
				bbcode.insertAtCursor("[quote]" + str_clipboard + "[/quote]");
				break;
			
			case "img":
				bbcode.insertAtCursor("[img]" + str_clipboard + "[/img]");
				break;
			
			case "url":
				bbcode.insertAtCursor("[url]" + str_clipboard + "[/url]");
				break;
			case "urlclip":
				if(!this.BBCode_use_newurlclip) {
					bbcode.insertAtCursorEx("[url=\"" + str_clipboard + "\"]" + str, "[/url]", select_flag);
				}else{
					if(str != "") {
						bbcode.insertAtCursorEx("[url=\"" + str_clipboard + "\"]" + str, "[/url]", select_flag);
					}else{
						bbcode.insertAtCurso("[url]" + str_clipboard + "[/url]");
					}
				}
				break;
			
			case "mail":
				bbcode.insertAtCursorEx("[email]" + str_clipboard, "[/email]", select_flag);
				break;
			case "mailclip":
				bbcode.insertAtCursorEx("[email=\"" + str_clipboard +"\"]" + str, "[/email]", select_flag);
				break;
			
			case "spoiler":
				bbcode.insertAtCursorEx("[spoiler]" + str, "[/spoiler]", select_flag);
				break;
			case "spoilerclip":
				bbcode.insertAtCursor("[spoiler]" + str_clipboard + "[/spoiler]");
				break;
				
			case "spoilerclip_theme":
				var spoiler_theme = this.prompt(bbcode.getBBCodeStringBundle.getString("bbcode.spoiler.theme"));
				if(spoiler_theme == "" || spoiler_theme == null){
					break;
				}else{
					bbcode.insertAtCursor("[spoiler=\"" + spoiler_theme + "\"]" + str_clipboard + "[/spoiler]");
				}
				break;
			
			case "strike":
				bbcode.insertAtCursorEx("[strike]" + str, "[/strike]", select_flag);
				break;
			
			case "right":
				bbcode.insertAtCursorEx("[right]" + str, "[/right]", select_flag);
				break;
			
			case "left":
				bbcode.insertAtCursorEx("[left]" + str, "[/left]", select_flag);
				break;
			
			case "blockquote":
				bbcode.insertAtCursorEx("[indent]" + str, "[/indent]", select_flag);
				break;
			
			case "center":
				bbcode.insertAtCursorEx("[center]" + str, "[/center]", select_flag);
				break;
			
			case "bold":
				bbcode.insertAtCursorEx("[b]" + str, "[/b]", select_flag);
				break;
			
			case "italic":
				bbcode.insertAtCursorEx("[i]" + str, "[/i]", select_flag);
				break;
			
			case "underline":
				bbcode.insertAtCursorEx("[u]" + str, "[/u]", select_flag);
				break;
			
			case "code":
				bbcode.insertAtCursorEx("[code]" + str, "[/code]", select_flag);
				break;
			
			case "codeclip":
				bbcode.insertAtCursor("[code]" + str_clipboard + "[/code]");
				break;
			
			case "php":
				bbcode.insertAtCursorEx("[php]" + str, "[/php]", select_flag);
				break;		
			case "phpclip":
				bbcode.insertAtCursor("[php]" + str_clipboard + "[/php]");
				break;
			
			case "html":
				bbcode.insertAtCursorEx("[html]" + str, "[/html]", select_flag);
				break;
			case "htmlclip":
				bbcode.insertAtCursor("[html]" + str_clipboard + "[/html]");
				break;
			
			case "threadID":
				var id = this.prompt(bbcode.getBBCodeStringBundle.getFormattedString("bbcode.thread_post_id.BBCode", ["Threads"]));
				var idN = parseInt(id);
				if(id != null && id != '') {
					if(!isNaN(idN)) {
						if(str != "") {
							bbcode.insertAtCursor("[thread=\"" + idN + "\"]" + str + "[/thread]");
						}else{
							bbcode.insertAtCursor("[thread]" + idN + "[/thread]");
						}
					}else{
						alertText = bbcode.getBBCodeStringBundle.getString("bbcode.list.err") + bbcode.getBBCodeStringBundle.getFormattedString("bbcode.NaN.err", [id]);
						alert(alertText);
					}
				}
				break;
			case "threadIDclip":
				if(str != "") {
					bbcode.insertAtCursorEx("[thread=\"" + str_clipboard + "\"]" + str, "[/thread]", select_flag);
				}else{
					bbcode.insertAtCursorEx("[thread]" + str_clipboard, "[/thread]", select_flag);
				}
				if(isNaN(parseInt(str_clipboard))) {
					alertText = bbcode.getBBCodeStringBundle.getFormattedString("bbcode.thread_post_clip.warning", [str_clipboard, "Threads"]);
					alert(alertText);
				}
				break;
			
			case "postID":
				var id = this.prompt(bbcode.getBBCodeStringBundle.getFormattedString("bbcode.thread_post_id.BBCode", ["Posts"]));
				var idN = parseInt(id);
				if(id != null && id != '') {
					if(!isNaN(idN)) {
						if(str != "") {
							bbcode.insertAtCursor("[post=\"" + idN + "\"]" + str + "[/post]");
						}else{
							bbcode.insertAtCursor("[post]" + idN + "[/post]");
						}
					}else{
						alertText = bbcode.getBBCodeStringBundle.getString("bbcode.list.err") + bbcode.getBBCodeStringBundle.getFormattedString("bbcode.NaN.err", [id]);
						alert(alertText);
					}
				}
				break;
			case "postIDclip":
				if(str != "") {
					bbcode.insertAtCursorEx("[post=\"" + str_clipboard + "\"]" + str, "[/post]", select_flag);
				}else{
					bbcode.insertAtCursorEx("[post]" + str_clipboard, "[/post]", select_flag);
				}
				if(isNaN(parseInt(str_clipboard))) {
					alertText = bbcode.getBBCodeStringBundle.getFormattedString("bbcode.thread_post_clip.warning", [str_clipboard, "Posts"]);
					alert(alertText+"\n"+str_clipboard);
				}
				break;
			
			case "highlight":
				bbcode.insertAtCursorEx("[highlight]" + str, "[/highlight]", select_flag);
				break;
			
			case "list":
				var listElements = " ";
				var listString = "";
				while(true){
					var listSort = this.prompt(bbcode.getBBCodeStringBundle.getString("bbcode.listtype.BBCode"));
					if((listSort == "a") || (listSort == "A") || (listSort == 1) || (listSort == "") || (listSort == null)){
						break;
					}else{
						alert(bbcode.getBBCodeStringBundle.getString("bbcode.list.err")+" \""+listSort+"\"");
						continue;
					}
				}
				while(listElements != "" && listElements != null){
					alertText = bbcode.getBBCodeStringBundle.getString("bbcode.list");
					listElements = this.prompt(alertText);
					if(listElements != "" && listElements != null ){
						listString = listString + "\n[*]" +listElements;
					}
				}
				if(listElements != null && listString != ""){
					if(listSort != ""){
						bbcode.insertAtCursor("[list=\"" + listSort + "\"]" + listString + "[/list]");
					}else{
						bbcode.insertAtCursor("[list]" + listString + "[/list]");
					}
				}
				break;
			
			case "size":
				var re = [];
				var bbcodesize = [BBCode_Size];
				window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize", "chrome,modal,centerscreen", re, bbcodesize);
				if(re[0]){
					bbcode.insertAtCursorEx("[size=\"" + bbcodesize[0] + "\"]" + str, "[/size]", select_flag);
					BBCode_Size = bbcodesize[0];
				}
				break;
			
			case "color":
				var re = [];
				var bbcoedcolor = [BBCode_Color];
				window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor", "chrome,modal,centerscreen", re, bbcoedcolor);
				if(re[0]){
					bbcode.insertAtCursorEx("[color=\"" + bbcoedcolor[0] + "\"]" + str, "[/color]", select_flag);
					BBCode_Color = bbcoedcolor[0];
				}
				break;
			
			case "justify":
				bbcode.insertAtCursorEx("[justify]" + str, "[/justify]", select_flag);
				break;
			
			case "sub":
				bbcode.insertAtCursorEx("[sub]" + str, "[/sub]", select_flag);
				break;
			
			case "sup":
				bbcode.insertAtCursorEx("[sup]" + str, "[/sup]", select_flag);
				break;
				
			case "attach":
				bbcode.insertAtCursorEx("[attach]" + str, "[/attach]", select_flag);
				break;
				
			case "attachclip":
				bbcode.insertAtCursorEx("[attach]" + str_clipboard, "[/attach]", select_flag);
				break;
			
			case "custom":
				var dateTimeService = Components.classes["@mozilla.org/intl/scriptabledateformat;1"].getService(Components.interfaces.nsIScriptableDateFormat);
				var timeStamp = new Date();
				var date = dateTimeService.FormatDate("", dateTimeService.dateFormatShort, timeStamp.getFullYear(), timeStamp.getMonth()+1, timeStamp.getDate());
				var time = dateTimeService.FormatTime("", dateTimeService.timeFormatNoSeconds, timeStamp.getHours(), timeStamp.getMinutes(), timeStamp.getSeconds());
				var tempValue = unescape(this.getPrefBBCodeCompose.getLocalizedUnicharPref('bbcode.tags.custom.' + myCustomStringIndex + '.url'));
				tempValue = this.replaceText(tempValue, "_date_", date);
				tempValue = this.replaceText(tempValue, "_time_", time);
				tempValue = this.replaceText(tempValue, "_value_", str);
				tempValue = this.replaceText(tempValue, "_clipboard_", str_clipboard);
				bbcode.insertAtCursorEx(tempValue, "", select_flag);
				break;
			
			case "syntaxjs":
				theCode = this.syntaxByType("js", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			case "syntaxcss":
				theCode = this.syntaxByType("css", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			case "syntaxxml":
				theCode = this.syntaxByType("xml", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			//End BBCODE
			default: 
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.option.err");
				alert(alertText);
		}
	},

	insertAtCursorSetupHTML: function(myCommand, str_clipboard, str, theBox) {
		var bold = "<b>";
		var boldend="<\/b>";
		var color = "<font color=\"";
		var color2 = "\">";
		var colorend = "<\/font>";
		var colorend2 = "</font>";
		var select_flag = (str != "");
		var alertText = "";
		var theCode = "";
		switch (myCommand){
			case "img":
				bbcode.insertAtCursor("<img src=\"" + str_clipboard + "\"/>");
				break;
			
			case "imgalt":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.img.alttext");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursor("<img src=\"" + str_clipboard + "\"" + " alt=\"" + theCode + "\"/>");
				break;
			
			case "mail":
				bbcode.insertAtCursor("<a href=\"mailto:" + str_clipboard + "\">" + str_clipboard + "</a>");
				break;
			
			case "mailclip":
				bbcode.insertAtCursorEx("<a href=\"mailto:" + str_clipboard + "\">" + str, "</a>", select_flag);
				break;
			
			case "url":
				bbcode.insertAtCursorEx("<a href=\"\">" + str, "</a>", select_flag);
				break;
			
			case "urltitle":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.url.title");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursorEx("<a href=\"\" title=\""+ theCode + "\">" + str, "</a>", select_flag);
				break;
			
			case "urlclip":
				bbcode.insertAtCursorEx("<a href=\"" + str_clipboard + "\">" + str, "</a>", select_flag);
				break;
			
			case "urlcliptitle":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.url.title");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursorEx("<a href=\"" + str_clipboard + "\" title=\""+ theCode + "\">" + str, "</a>", select_flag);
				break;
			
			case "bold":
				bbcode.insertAtCursorEx(bold+ str, boldend, select_flag);
				break;
			
			case "italic":
				bbcode.insertAtCursorEx("<i>" + str, "</i>", select_flag);
				break;
			
			case "underline":
				bbcode.insertAtCursorEx("<u>" + str, "</u>", select_flag);
				break;
			
			case "strike":
				bbcode.insertAtCursorEx("<strike>" + str, "</strike>", select_flag);
				break;
			
			case "blockquote":
				bbcode.insertAtCursorEx("<blockquote>" + str, "</blockquote>", select_flag);
				break;
			
			case "list":
				var listElements = " ";
				var listString = "";
				while(listElements != "" && listElements != null){ 
					alertText = bbcode.getBBCodeStringBundle.getString("bbcode.list");
					listElements = this.prompt(alertText);
					if(listElements != "" && listElements != null ){
						listString = listString + "\n<li>" +listElements + "</li>";
					}
				}
				if(listElements != null){
					bbcode.insertAtCursor("<ul>"  + listString + "</ul>");
				}
				break;
			
			case "size":
				var re = [];
				var bbcodesize = [BBCode_Size];
				window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize", "chrome,modal,centerscreen",re,bbcodesize);
				if(re[0]){
					bbcode.insertAtCursorEx("<font size=\"" + bbcodesize[0] + "px\">" + str, "</font>", select_flag);
					BBCode_Size = bbcodesize[0];
				}
				break;
			
			case "color":
				var re = [];
				var bbcoedcolor = [BBCode_Color];
				window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor", "chrome,modal,centerscreen",re,bbcoedcolor);
				if(re[0]){
					bbcode.insertAtCursorEx("<font color=\"" + bbcoedcolor[0] + "\">" + str, "</font>", select_flag);
					BBCode_Color = bbcoedcolor[0];
				}
				break;
			
			case "syntaxjs":
				theCode = this.syntaxByType("js", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
				
			case "syntaxcss":
				theCode = this.syntaxByType("css", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			case "syntaxxml":
				theCode = this.syntaxByType("xml", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			default : 
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.option.err");
				alert(alertText);
		}
	},
	
	insertAtCursorSetupXHTML: function(myCommand, str_clipboard, str, theBox) {
		var bold = "<strong>";
		var boldend="<\/strong>";
		var color = "<span style=\"color: ";
		var color2 = "\">";
		var colorend = "<\/span>";
		var colorend2 = "</span>";
		var select_flag = (str != "");
		var alertText = "";
		var theCode = "";
		switch (myCommand){
			case "img":
				bbcode.insertAtCursor("<img src=\"" + str_clipboard + "\"/>");
				break;
			
			case "imgalt":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.img.alttext");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursor("<img src=\"" + str_clipboard + "\"" + " alt=\"" + theCode + "\"/>");
				break;
			
			case "mail":
				bbcode.insertAtCursor("<a href=\"mailto:" + str_clipboard + "\">" + str_clipboard + "</a>");
				break;
			
			case "mailclip":
				bbcode.insertAtCursorEx("<a href=\"mailto:" + str_clipboard + "\">" + str, "</a>", select_flag);
				break;
			
			case "url":
				bbcode.insertAtCursorEx("<a href=\"\">" + str, "</a>", select_flag);
				break;
			
			case "urltitle":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.url.title");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursorEx("<a href=\"\" title=\""+ theCode + "\">" + str, "</a>", select_flag);
				break;
			
			case "urlclip":
				bbcode.insertAtCursorEx("<a href=\"" + str_clipboard + "\">" + str, "</a>", select_flag);
				break;
			
			case "urlcliptitle":
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.url.title");
				theCode = this.prompt(alertText);
				bbcode.insertAtCursorEx("<a href=\"" + str_clipboard + "\" title=\""+ theCode + "\">" + str, "</a>", select_flag);
				break;
			
			case "bold":
				bbcode.insertAtCursorEx("<strong>" + str, "</strong>", select_flag);
				break;
			
			case "italic":
				bbcode.insertAtCursorEx("<em>" + str, "</em>", select_flag);
				break;
			
			case "underline":
				bbcode.insertAtCursorEx("<ins>" + str, "</ins>", select_flag);
				break;
			
			case "strike":
				bbcode.insertAtCursorEx("<del>" + str, "</del>", select_flag);
				break;
			
			case "blockquote":
				bbcode.insertAtCursorEx("<blockquote>" + str, "</blockquote>", select_flag);
				break;
			
			case "list":
				var listElements = " ";
				var listString = "";
				while(listElements != "" && listElements != null){ 
					alertText = bbcode.getBBCodeStringBundle.getString("bbcode.list");
					listElements = this.prompt(alertText);
					if(listElements != "" && listElements != null ){
						listString = listString + "\n<li>" +listElements + "</li>";
					}
				}
				if(listElements != null){
					bbcode.insertAtCursor("<ul>"  + listString + "</ul>");
				}
				break;  
			
			case "size":
				var re = [];
				var bbcodesize = [BBCode_Size];
				window.openDialog("chrome://bbcode/content/bbcodesize.xul", "BBCodeSize", "chrome,modal,centerscreen",re,bbcodesize);
				if(re[0]){
					bbcode.insertAtCursorEx("<span style=\"font-size: " + bbcodesize[0] + "px\">" + str, "</span>", select_flag);
					BBCode_Size = bbcodesize[0];
				}
				break;
			
			case "color":
				var re = [];
				var bbcoedcolor = [BBCode_Color];
				window.openDialog("chrome://bbcode/content/bbcodecolor.xul", "BBCodeColor", "chrome,modal,centerscreen",re,bbcoedcolor);
				if(re[0]){
					bbcode.insertAtCursorEx("<span style=\"color: " + bbcoedcolor[0] + "\">" + str, "</span>", select_flag);
					BBCode_Color = bbcoedcolor[0];
				}
				break;
			
			case "syntaxjs":
				theCode = this.syntaxByType("js", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
				
			case "syntaxcss":
				theCode = this.syntaxByType("css", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			case "syntaxxml":
				theCode = this.syntaxByType("xml", str, bold, boldend, color, color2, colorend, colorend2);
				bbcode.insertAtCursor(theCode);
				break;
			
			default : 
				alertText = bbcode.getBBCodeStringBundle.getString("bbcode.option.err");
				alert(alertText);
			}
	},
  
	//Thanks to Nick (asquella) for pointing out this function to me
	insertAtCursor: function( aText) {
		try {
			var command = "cmd_insertText";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				params.setStringValue("state_data", aText);
				controller.doCommandWithParams(command, params);
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
	//Original Function by Patrick Wildenborg
	insertAtCursorEx: function(aText, bText, selection) {
		this.insertAtCursor(aText+bText);
		if(selection) {
			this.reselectCursor(aText+bText);
		} else {
			this.backupCursor(bText);
		}  
	},
	//Original Function by Asquella, modified by Patrick Wildenborg
	backupCursor: function(aText) {
		try {
			var command = "cmd_charPrevious";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				var len = aText.length
				while( len-- > 0 ) {
					controller.doCommand(command, params);
				}
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
	//Original Function by Asquella, modified by Patrick Wildenborg
	reselectCursor: function(aText) {
		try {
			var command = "cmd_selectCharPrevious";
			var controller = document.commandDispatcher.getControllerForCommand(command);
			if (controller && controller.isCommandEnabled(command)) {
				controller = controller.QueryInterface(Components.interfaces.nsICommandController);
				var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
				params = params.createInstance(Components.interfaces.nsICommandParams);
				var len = aText.length
				while( len-- > 0 ) {
					controller.doCommand(command, params);
				}
			}
		}catch (e) {
			dump("Can't do cmd_insertText! ");
			dump(e+"\n")
		}
	},
  
	OpenSettings: function() {
		//var fromEM = false;	// was used as 4th parameter
		window.openDialog('chrome://bbcode/content/bbcode_settings.xul', '&bbcode.settings.window;', 'chrome,modal,centerscreen,resizable', false, null);
	},
	
	_BBCodeStringbundle: null,
	get getBBCodeStringBundle() {
		if(!bbcode._BBCodeStringbundle){
			try{
				bbcode._BBCodeStringbundle = document.getElementById("bbcstrings");
			}catch (e){
			}
		}
		return bbcode._BBCodeStringbundle;
	},
	
	_BBCodePreferences: null,
	get getPrefBBCodeCompose() {
		if(!bbcode._BBCodePreferences){
			try{
				bbcode._BBCodePreferences = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
			}catch (e){
			}
		}
		return bbcode._BBCodePreferences;
	},
	
	//Set Defaults
	SetDefaultPrefs: function() {
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.bbcode", true);
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.3dcenter", true);
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.xhtml", true);
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.html", false);
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.symbol", false);
		this.getPrefBBCodeCompose.SetDefaultBoolPref("bbcode.contextmenu.custom", true);
		this.getPrefBBCodeCompose.SetDefaultCharPref("bbcode.font.color", "#000000");
		this.getPrefBBCodeCompose.SetDefaultCharPref("bbcode.font.size", "12");
		this.getPrefBBCodeCompose.SetDefaultIntPref("bbcode.tags.custom.number", 0);
		//Examples
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.1.label", "i.e.  Sig");
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.1.url", "-This is my Sig");
		
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.2.label", "i.e. Highlited Text");
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.2.url", "<html> _value_ </html>");
		
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.3.label", "i.e. highlighted text and clipboard");
		this.getPrefBBCodeCompose.SetDefaultUnicharPref("bbcode.tags.custom.3.url", "<pre class=_clipboard_> _value_ </pre>");
	},
	GetUserPrefs: function() {
		BBCode_Size = this.getPrefBBCodeCompose.GetCharPref("bbcode.font.size");
		BBCode_Color = this.getPrefBBCodeCompose.GetCharPref("bbcode.font.color");
		BBCode_enableBBCode = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.bbcode");
		BBCode_enable3dcenter = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.3dcenter");
		BBCode_enableXHTML = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.xhtml");
		BBCode_enableHTML = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.html");
		BBCode_enableSYMBOL = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.symbol");
		BBCode_enableCustomCode = this.getPrefBBCodeCompose.GetBoolPref("bbcode.contextmenu.custom");
		try{
			this.BBCode_use_newurlclip = this.getPrefBBCodeCompose.GetBoolPref("bbcode.use.newurlclip");
		}catch(e){
		}
	},
  
	PopulateCustomImages: function(event){
		var theNumber = this.getPrefBBCodeCompose.GetIntPref('bbcode.tags.custom.number');
		var bbc_MainMenu = document.getElementById("bbcode_ct_custom_menu");
		if (theNumber >= 1) {
			// clear the menu before adding them
			bbcode.ClearMenu(event.target);
			//Add custom Entries
			for (i=1; i <= theNumber; ++i) {
				var bcCustomLabel = unescape(this.getPrefBBCodeCompose.getLocalizedUnicharPref('bbcode.tags.custom.'+ i + '.label'));
				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", bcCustomLabel + " ");		// Add a space at the end to prevent theme conflicts (like profiles named "home")
				menuitem.setAttribute("id", "context-bbc-custom-" + i);
				menuitem.setAttribute("oncommand", "bbcode.bbcode('custom', 'bbcode', " + i + ");");
				bbc_MainMenu.appendChild(menuitem);
			}
		}else{
			// If changed to Zero again, erase
			bbcode.ClearMenu(event.target);
		}
		return true;
	},
	ClearMenu: function(aParent) {
		//jed, your old version did not remove all entries
		while(aParent.hasChildNodes()) {
			aParent.removeChild(aParent.lastChild);
		}
	},
	
	// The following two functions are modified versions of the Syntax highlighter for JavaScript (1.2) script found at http://www.howtocreate.co.uk.
	inAr: function(oStr) {
		var opAr = new Array('+','-','*','%','=','+=','-=','&gt;','&lt;','&gt;=','&lt;=','++','--','==','!=','&lt;&lt;','&gt;&gt;','&amp;','|','^','~','!','&amp;&amp;','||','?',':');
		for( var x = 0; x < opAr.length; x++ ) {
			if( oStr.substr(0,opAr[x].length) == opAr[x] ) {
				return opAr[x];
			}
		}
		return false;
	},
	
	//replaceText(inString,oldText,newText);
	replaceText: function(inString, oldText, newText) {
		//return (inString.split(oldText).join(newText));
		return inString.replace(oldText, newText);
	},
	
	syntaxByType: function(syntaxType, tForm, bold, boldend, color, color2, colorend, colorend2) { //start conversion by preparing variables
		var returnValue = bbcodeSyntax(syntaxType, tForm, bold, boldend, color, color2, colorend, colorend2);
		return returnValue;
	}
}


//Make sure bbcode loads on start-up of browser.
window.addEventListener('load', bbcode.init, false); 
